import java.io.*;
import java.text.*; // streamtokenizer

import java.lang.ClassNotFoundException;
import java.lang.InstantiationException;
import java.lang.IllegalAccessException;

import chocobar.GenericChocoGraph;
import graph.GenericGraph;
import conf.ChocoConfig;
import jeu.Strategy;
import jeu.Simulation;

/* command line parsing */
import com.martiansoftware.jsap.*;
import com.martiansoftware.jsap.stringparsers.*;


public final class ChocoBarLite 
{
	/**
	 * Fonction de test de reprsentation de graphe 
	 **/
	public static void main(String[] args) throws Exception {

		JSAP jsap = new JSAP();

		FlaggedOption opt;
		Switch sw;

		sw = new Switch("graphgen")  // true = gnration du graphe  partir de la taill vrif
			.setShortFlag('g')      // false = on prend le graphe du fichier en entre
			.setLongFlag("graph");
		sw.setHelp("Gnrer le graphe sans fichier d'entre.");
		jsap.registerParameter(sw);
		
		sw = new Switch("strategy")  // true = on gere la stratgie
			.setShortFlag('s')      // false = on ne gre pas la stratgie
			.setLongFlag("strategy");
		sw.setHelp("Gnration des stratgies. (il faut obligatoirement activer la gnration de graphe avec ou sans fichiers)");
		jsap.registerParameter(sw); 
		
		sw = new Switch("simulate")  // true = on gere la simulation
			.setShortFlag('m')      // false = on ne gre pas la stratgie
			.setLongFlag("simulate");
		sw.setHelp("Gestion de la partie simulation. (implique -s)");
		jsap.registerParameter(sw); 
		
		sw = new Switch("verbose")   // true = mode verbeux 
			.setShortFlag('v')      // false = normal
			.setLongFlag("verbose");
		sw.setHelp("Mode verbeux");
		jsap.registerParameter(sw);
		
		opt= new FlaggedOption("cbmodel")  // type de graphe
			.setStringParser(new StringStringParser())
			.setDefault("bpl.ChocoBar")
			.setRequired(false)
			.setLongFlag("cbmodel");
		opt.setHelp("Modle de ChocoBar bpl.ChocoBar, combi.ChocoBarLong, combi.ChocoBarBigInt, combi.ChocoBarArray. (dfaut: bpl.ChocoBar)");
		jsap.registerParameter(opt);
		
		opt= new FlaggedOption("cbstyle")  // type de graphe
			.setStringParser(new StringStringParser())
			.setDefault("bpl")
			.setRequired(false)
			.setLongFlag("cbstyle");
		opt.setHelp("Style de ChocoBar : bpl, combi. (dfaut: combi)");
		jsap.registerParameter(opt);

		opt= new FlaggedOption("edgemodel")  // type de graphe
			.setStringParser(new StringStringParser())
			.setDefault("EdgeBigIntModel")
			.setRequired(false)
			.setLongFlag("edgemodel");
		opt.setHelp("Modle d'Arc (edge) : EdgeBigIntModel, EdgeBitSetModel, EdgeListModel, EdgeMatrixModel.");
		jsap.registerParameter(opt);

	
		opt = new FlaggedOption("width")   // largeur du chocobar
			.setStringParser(new IntegerStringParser())
			.setDefault("6") 
			.setRequired(false) 
			.setShortFlag('w') 
			.setLongFlag("width");
		opt.setHelp("Largeur de la ChocoBar. (dfaut 6)");
		jsap.registerParameter(opt);
		
		opt = new FlaggedOption("height")  // hauteur du chocobar
			.setStringParser(new IntegerStringParser())
			.setDefault("6") 
			.setRequired(false) 
			.setShortFlag('h') 
			.setLongFlag("height");
		opt.setHelp("Hauteur de la ChocoBar. (dfaut 6)");
		jsap.registerParameter(opt);

		opt= new FlaggedOption("ginput")  // fichier de graphe en entre
			.setStringParser(new StringStringParser())
			.setDefault("")
			.setRequired(false)
			.setShortFlag('G')
			.setLongFlag("gpinput");
		opt.setHelp("Fichier de graphe en entre.");
		jsap.registerParameter(opt);


		opt= new FlaggedOption("strategyA")  // fichier de graphe en entre
			.setStringParser(new StringStringParser())
			.setDefault("")
			.setRequired(false)
			.setShortFlag('A')
			.setLongFlag("strategyA");
		opt.setHelp("Fichier de stratgie pour le joueur A. (Elle sera gnre au besoin sinon)");
		jsap.registerParameter(opt);
		
		opt= new FlaggedOption("strategyB")  // fichier de graphe en entre
			.setStringParser(new StringStringParser())
			.setDefault("")
			.setRequired(false)
			.setShortFlag('B')
			.setLongFlag("strategyB");
		opt.setHelp("Fichier de stratgie pour le joueur B. (Elle sera gnre au besoin sinon)");
		jsap.registerParameter(opt);
		
		sw = new Switch("random")  // La stratgie est alatoire 
			.setShortFlag('r')
			.setLongFlag("random");
		sw.setHelp("En cas de gnration, le joueur A aura une statgie alatoire. (Par dfault elle est gagnante");
		jsap.registerParameter(sw);
		
		JSAPResult config = jsap.parse(args);    
		if (!config.success()) {
			System.err.println();
			for (java.util.Iterator errs = config.getErrorMessageIterator();
					errs.hasNext();) {
				System.err.println("Error: " + errs.next());
			}
			System.err.print("Usage: ChocoBarLite ");
			System.err.println(jsap.getUsage());
			System.err.println();
			// show full help as well
			System.err.println(jsap.getHelp());
			System.err.println();
			System.exit(1);
		} else {
			/* la ligne de commande est valide, on initialise donc les valeurs */
			ChocoConfig.size = config.getInt("width");
			ChocoConfig.height = config.getInt("height");
			ChocoConfig.chocoBarModel = config.getString("cbmodel");
			ChocoConfig.chocoBarStyle = config.getString("cbstyle");
			ChocoConfig.edgeModel = config.getString("edgemodel");
			
			GenericChocoGraph c=null;
			try {
				// Recherche du bon ChocoGraph
				Class cCG = Class.forName("chocobar."+ChocoConfig.chocoBarStyle+".ChocoGraph");
				c = (GenericChocoGraph) cCG.newInstance();
			} catch (Exception e){
				System.err.println("ERREUR: Le modle de graphe '"+ChocoConfig.chocoBarStyle+"' est inconnu.");
				e.printStackTrace();
				System.exit(1);
			}
			try {
				GenericGraph g=null; 
				if (config.getBoolean("graphgen")){
					g = c.genChoco(true);
				} else {
					if (!config.getString("ginput").equals("")){ // on vrifie si la chaine n'est pas vide...
						// On instancie le bon Graph
						Class cCG = Class.forName("chocobar."+ChocoConfig.chocoBarStyle+".Graph");
						g = (GenericGraph) cCG.newInstance();
						g.loadFromFile(config.getString("ginput"));
					}
				}
				if (g!=null){
					System.out.println("Fin de calcul du graphe");
				System.out.println(g.toDot());
					
				Strategy a=null;
				Strategy b=null;
				if (config.getBoolean("strategy") || config.getBoolean("simulate")) {
					// On a demand  grer la partie stratgie

					// Cration de la stratgie de A
					a = new Strategy();

					// A a-t-il une stratgie dfinie ?
					if ( config.getString("strategyA").equals("") ) {
						// Non. La stratgie de A est-elle alatoire ?
						if (config.getBoolean("random")) {
							a.constructRandomStrategy(g);
						} else {
							a.constructWinningStrategy(g);
						}
					} else {
						// Oui, on lit donc la stratgie du fichier
						a.constructFromFile(config.getString("strategyA"));
					}
					
					// Cration de la stratgie de B
					b = new Strategy();

					// B a-t-il une stratgie dfinie ?
					if ( config.getString("strategyB").equals("") ) {
						// B a forcment une statgie alatoire
						b.constructRandomStrategy(g);
					} else {
						// Oui, on lit donc la stratgie du fichier
						b.constructFromFile(config.getString("strategyB"));
					}
				
					// On affiche la stratgie de A
					a.display();
				}
					
				if (config.getBoolean("simulate")) {
					// On a demand  grer la partie simulate
					// A et B ont dj une stratgie (-m implique -s)
					Simulation.simulate(a, b, g);
				}
				} else {
					System.err.println("ERREUR: Aucun graphe gnr. Impossible d'aller plus loin");
					System.err.println();
					System.err.print("Usage: ChocoBarLite ");
					System.err.println(jsap.getUsage());
					System.err.println();
					// show full help as well
					System.err.println(jsap.getHelp());
					System.err.println();
					System.exit(1);

				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

	}

}
