package chocobar.combi;

import java.util.Vector;

import conf.ChocoConfig;

class ChocoBarArray extends ChocoBar {
	char points[];

	/**
	 * Constructeur d'une barre de choco ayant x carres en largeur
	 * et y carres en hauteur, toute neuve.
	 **/
	private ChocoBarArray() 
	{
		points = new char[size];
		for (int i=0; (i<size); i++) {
			points[i] = (char) height;
		}
	}

	/** Constructeur par copie.
	 */
	private ChocoBarArray(ChocoBarArray old)
	{
		points = new char[size];
		for (int i=0; (i<size); i++) {
			points[i] = old.points[i];
		}
	}

	/** Constructeur par initialisation
	 */
	private ChocoBarArray(char p[])
	{
		points = new char[size];
		for (int i=0; (i<size); i++) {
			points[i] = p[i];
		}
	}

	/**
	 * Affichage de la barre de choco en ligne.
	 **/
	public String toLine()
	{
		String s = "";
		for (int i=0; (i<size); i++) {
			s += (int)points[i] + " ";
		}
		return s;
	}
	
	public String toString()
	{
		return toLine();
	}
	
	/**
	 * Vrifie si le carre (x, y) est dj mang.
	 **/
	protected boolean isBroken(int cX, int cY)
	{
		// We are inside what's defined
		return ((height - cY) <= points[cX]);
	}

	/**
	 * Comparaison de barres.
	 * La conversion entre type de ChocoBar n'est pas implmente 
	 **/
	public boolean equals(ChocoBar comp) { return equals((ChocoBarArray) comp); }
	public boolean equals(ChocoBarArray comp)
	{
		// If the other bar is null we're not equal, since
		// we're obviously not null ;-)
		if (comp==null) {
			return false;
		} 

		// not the same size, isn't the same
		if (comp.size != size || comp.height != height) {
			return false;	
		}
		
		// We don't have a choice now... doing it the hard way	
		for (int i=0; (i<size); i++) {
			if (comp.points[i] != points[i]) {
				return false;
			}
		}

		// If we arrive here... it's the same !
		return true;
	}

	/** Recherche si l'on peut tre un fils de la ChocoBar.
	 * La conversion entre type de ChocoBar n'est pas implmente 
	 */
	public boolean isNextOf(ChocoBar parent) { return isNextOf((ChocoBarArray) parent); }
	public boolean isNextOf(ChocoBarArray parent)
	{
		int eaten = -1;
		for (int i=0; (i<size); i++) {
			// Si le parent est plus mang que le fils, c'est toujours non !
			if (parent.points[i] > points[i]) {
				return false;
			}
			
			if (eaten == -1) {
				if (parent.points[i] != points[i]) {
					// On a mang ce carr, les carrs suivants doivent donc tre
					// identiquent  celui-ci ou  celui du pre
					eaten = points[i];
					// On a trait ce carr, on passe au suivant
					continue;
				}
				// Le carr est identique, suivant !
				continue;
			}
			
			// On a mang le carr prcdent, ils doivent donc tre 
			// identiques au point prcdents, ou  celui que l'on vient de
			// manger  
			if (parent.points[i] != points[i] && points[i] != eaten) {
				return false;
			}
		}

		// Si on arrive ici, c'est que tout c'est bien pass, a-t-on un carr
		// mang ?
		return (eaten != -1);
	}

	/** Usine pour construire une liste de ChocoBar.
	 * Cette usine contruit une liste exhaustive et ordonne de ChocoBar.
	 */
	public static ChocoBar[] genChocoBars(int x, int y)
	{
		// On mets  jour les variables statiques de taille
		ChocoBarArray.size = x; 
		ChocoBarArray.height = y; 
		
		// Calcul du nombre de chocobars diffrentes existantes
		int nbChocobars = nbChocoBars(x, y);
		
		// Hop, on rserve le tableau de rsultat...
		ChocoBar[] chocoBars = new ChocoBar[nbChocobars];
		
		// Il est temps de remplir le tableau !
		
		// Cration du tableau pour stocker les valeurs
		char[] points = new char[x];

		boolean fin = false;
		int cbIdx = 0;	
		while (!fin) {
			// Affectation
			chocoBars[cbIdx] = new ChocoBarArray(points);
			cbIdx++;

			// Calcul du suivant :
			
			// On incrmente ... 
			int idx = x-1;
			points[idx] ++;

			// ... et on gre la retenue
			boolean overflow = (points[idx] > y);
			while (points[idx] > y) {
				if (idx == 0) {
					fin = true;
					break;
				}
				points[idx-1] ++;
				idx--;
			}
			// On rpercute la modif vers la droite s'il y a eu overflow
			if (overflow) {
				for (int i=idx; (i<x); i++) {
					points[i] = points[idx];
				}
			}
		}
		
		return chocoBars;
	}

	public static ChocoBar[] genChocoBars(Integer x, Integer y)
	{
		return genChocoBars(x.intValue(), y.intValue());
	}
	

	// Implmentation de Vertice
  
	public void setLabel(String l) { }
	public String getLabel() { return "null"; }
	public void setColor(int c) { }
	public int getColor() { return 0; }
}
